import json
from os import path
from time import time

class JsonCacher(object):
    """
        Simple cacher using a single json file.

        file_path   -- The file to store database information in. (eg. './db.json')
        refresh_age -- The youngest cached item we should refresh in hours.
        api_url     -- The url leading every api call. (eg. 'http://thebluealliance.com/api/v2')
    """
    def __init__(self, file_path, refresh_age, api_url):
        self.file_path = file_path
        self.refresh_age = refresh_age * 60 * 60 # Convert from hours to seconds.
        self.api_url = api_url
        self._check_file()

    """
        Retrieve the cached data for a url.

        url -- The url of the requested data.
        return -- The data, or None if no data has been cached.
    """
    def access(self, url):
        obj = self._get(url)
        if "data" in obj.keys():
            return obj["data"]
        else:
            return None

    """
        Offer some data to the cacher.
        url  -- The url source for the data.
        data -- The data to cache at the url.
        return -- True if the cacher accepted the data, otherwise False
    """
    def offer(self, url, data):
        cached_obj = self._get(url)
        if len(cached_obj.keys()) < 1 or time() - float(cached_obj["time"]) >= self.refresh_age:
            self._add(url, data)
            return True
        return False

    def _check_file(self):
        if not path.isfile(self.file_path):
            open(self.file_path, "w+").write("{}")

    def _load_file(self, mode="r"):
        return open(self.file_path, mode)

    def _get(self, url):
        if type(url) == type(""):
            keys = self._check_url(url).split("/")
        db = json.load(self._load_file())
        return self._recursive_get(db, keys)

    def _add(self, url, data):
        keys = self._check_url(url).split("/")
        db = json.load(self._load_file())
        obj = self._recursive_get(db, keys[:-1])
        obj[keys[-1]] = {
            "data": data,
            "time": str(time())
        }
        json.dump(db, self._load_file('w+'))

    def _check_url(self, url):
        if(url.startswith(self.api_url)):
            url = url[len(self.api_url):]
        return url

    @staticmethod
    def _recursive_get(db, keys):
        obj = db
        for key in keys:
            if key not in obj.keys():
                obj[key] = {}
            obj = obj[key]
        return obj
